@@ -3,7 +3,7 @@
 # Base subroutines brought by the the Project-Builder project
 # which can be easily used by whatever perl project
 #
-# Copyright B. Cornec 2007-2012
+# Copyright B. Cornec 2007-2016
 # Eric Anderson's changes are (c) Copyright 2012 Hewlett Packard
 # Provided under the GPL v2
 #
@@ -134,7 +134,7 @@ Needs pb_log support, so pb_log_init should have been called before.
 
 The first parameter is the shell command to call. This command should NOT use redirections.
 The second parameter is the message to print on screen. If none is given, then the command is printed.
-The third parameter prints the result of the command after correct execution if value is "verbose". If value is "noredir", it avoids redirecting outputs (e.g. for vi). If value is "quiet", doesn't print anything at all. If value is "mayfail", failure of the command is ok even if $Global::pb_stop_on_error is set, because the caller will be handling the error. A "verbose" can be added to mayfail to have it explain why it failed
+The third parameter prints the result of the command after correct execution if value is "verbose". If value is "noredir", it avoids redirecting outputs (e.g. for vi). If value is "quiet", doesn't print anything at all. If value is "mayfail", failure of the command is ok even if $Global::pb_stop_on_error is set, because the caller will be handling the error. A "verbose" can be added to mayfail to have it explain why it failed. If value is verbose_PREF, then every output command will be prefixed with PREF.
 This function returns as a result the return value of the system command.
 
 If no error reported, it prints OK on the screen, just after the message. Else it prints the errors generated.
@@ -145,7 +145,7 @@ sub pb_system {
 
 my $cmd=shift;
 my $cmt=shift || $cmd;
-my $verbose=shift || undef;
+my $verbose=shift;
 my $redir = "";
 
 pb_log(0,"$cmt... ") if ((! defined $verbose) || ($verbose ne "quiet"));
@@ -162,7 +162,7 @@ my $res = $?;
 if ((defined $verbose) and ($verbose =~ /mayfail/)) {
 	pb_log(0,"NOT OK but non blocking\n") if ($res != 0);
 	pb_log(0,"OK\n") if ($res == 0);
-	pb_display_file("$ENV{'PBTMP'}/system.$$.log") if ((-f "$ENV{'PBTMP'}/system.$$.log") and (defined $verbose) and ($verbose =~ /verbose/));
+	pb_display_file("$ENV{'PBTMP'}/system.$$.log",undef,$verbose) if ((-f "$ENV{'PBTMP'}/system.$$.log") and ($verbose =~ /verbose/));
 	return($res) 
 }
 
@@ -174,7 +174,7 @@ $error = "ERROR: child ($cmd) cwd=$cwd exited with value ".($res >> 8)."\n" if (
 
 if (defined $error) {
 	pb_log(0, $error) if (((! defined $verbose) || ($verbose ne "quiet")) || ($Global::pb_stop_on_error));
-	pb_display_file("$ENV{'PBTMP'}/system.$$.log") if ((-f "$ENV{'PBTMP'}/system.$$.log") and ((! defined $verbose) || ($verbose ne "quiet") || $Global::pb_stop_on_error));
+	pb_display_file("$ENV{'PBTMP'}/system.$$.log",undef,$verbose) if ((-f "$ENV{'PBTMP'}/system.$$.log") and ((! defined $verbose) || ($verbose ne "quiet") || $Global::pb_stop_on_error));
 	if ($Global::pb_stop_on_error) {
 		confess("ERROR running command ($cmd) with cwd=$cwd, pid=$$");
 	} else {
@@ -182,7 +182,7 @@ if (defined $error) {
 	}
 } else {
 	pb_log(0,"OK\n") if ((! defined $verbose) || ($verbose ne "quiet"));
-	pb_display_file("$ENV{'PBTMP'}/system.$$.log") if ((-f "$ENV{'PBTMP'}/system.$$.log") and (defined $verbose) and ($verbose ne "quiet"));
+	pb_display_file("$ENV{'PBTMP'}/system.$$.log",undef,$verbose) if ((-f "$ENV{'PBTMP'}/system.$$.log") and (defined $verbose) and ($verbose ne "quiet"));
 }
 
 return($res);
@@ -200,7 +200,7 @@ Cf man URI.
 
 sub pb_get_uri {
 
-my $uri = shift || undef;
+my $uri = shift;
 
 pb_log(2,"DEBUG: uri:" . (defined $uri ? $uri : '') . "\n");
 my ($scheme, $authority, $path, $query, $fragment) =
@@ -251,8 +251,11 @@ The call to B<pb_log_init> is typically done after getting a parameter on the CL
 
 sub pb_log_init {
 
-$pbdebug = shift || 0;
-$pbLOG = shift || \*STDOUT;
+$pbdebug = shift;
+$pbLOG = shift;
+
+$pbdebug = 0 if (not defined $pbdebug);
+$pbLOG = \*STDOUT if (not defined $pbLOG);
 pb_log(1,"Debug value: $pbdebug\n");
 
 } 
@@ -277,9 +280,11 @@ Here is a usage example:
 
 sub pb_log {
 
-my $dlevel = shift || 0;
-my $msg = shift || "";
+my $dlevel = shift;
+my $msg = shift;
 
+$dlevel = 0 if (not defined $dlevel);
+$msg = "" if (not defined $msg);
 $pbLOG = \*STDOUT if (not defined $pbLOG);
 
 print $pbLOG "$msg" if ($dlevel <= $pbdebug);
@@ -289,8 +294,9 @@ print "$msg" if (($dlevel == 0) && ($pbLOG != \*STDOUT));
 
 =item B<pb_display_file>
 
-This function print the content of the file passed in parameter.
+This function prints the content of the file passed in parameter.
 If a second parameter is given, this is the descriptor of the logfile to write to in addtion to STDOUT.
+If a third parameter is given, this is the prefix providing it's writen as verbose_PREF. In which case the PREF string will be added before the line output.
 
 This is a cat equivalent function.
 
@@ -299,12 +305,20 @@ This is a cat equivalent function.
 sub pb_display_file {
 
 my $file=shift;
-my $desc=shift || undef;
+my $desc=shift;
+my $prefix=shift;
 
 return if (not -f $file);
 my $cnt = pb_get_content($file);
-print "$cnt\n";
-print $desc "$cnt\n" if (defined $desc);
+# If we have a prefix, then add it at each line
+if ((defined $prefix) and ($prefix =~ "_")) {
+	$prefix =~ s/verbose_//;
+	$cnt =~ s/(.*)\n/$prefix$1\n/g;
+} else {
+	$prefix = "";
+}
+print "$prefix$cnt";
+print $desc "$prefix$cnt" if (defined $desc);
 }
 
 =item B<pb_get_content>
@@ -312,18 +326,14 @@ print $desc "$cnt\n" if (defined $desc);
 This function returns the content of the file passed in parameter.
 
 =cut
-
 sub pb_get_content {
 
 my $file=shift;
 
-my $bkp = $/;
-undef $/;
 open(R,$file) || die "Unable to open $file: $!";
+local $/;
 my $content=<R>;
 close(R);
-chomp($content);
-$/ = $bkp;
 return($content);
 }
 
@@ -355,7 +365,8 @@ Fundtion to call before exiting pb so cleanup is done
 
 sub pb_exit {
 
-my $ret = shift || 0;
+my $ret = shift;
+$ret = 0 if (not defined $ret);
 pb_log(0,"Please remove manually $ENV{'PBTMP'} after debug analysis\n") if ($pbdebug > 1);
 exit($ret);
 }
@@ -420,6 +431,11 @@ This function initializes the environemnt variable PBTMP to a random value. This
 
 sub pb_temp_init {
 
+my $pbkeep = shift;	
+
+# Do not keep temp files by default
+$pbkeep = 0 if (not defined $pbkeep);
+
 if (not defined $ENV{'TMPDIR'}) {
 	$ENV{'TMPDIR'}="/tmp";
 }
@@ -442,9 +458,10 @@ if ($@) {
 	}
 	undef $template;
 	$template = pack "a" x @template, @template;
-	pb_mkdir_p("$ENV{'TMPDIR'}/$template");
+	$ENV{'PBTMP'} = "$ENV{'TMPDIR'}/$template";
+	pb_mkdir_p($ENV{'PBTMP'});
 } else {
-	if ($pbdebug > 1) {
+	if (($pbdebug > 1) || ($pbkeep == 1)) {
 		$ENV{'PBTMP'} = tempdir( "pb.XXXXXXXXXX", DIR => $ENV{'TMPDIR'});
 		pb_log(2,"DEBUG: Creating a non-volatile temporary directory ($ENV{'PBTMP'})\n");
 	} else {
@@ -496,9 +513,9 @@ The optional commands are passed as a comma separated string as second parameter
 
 sub pb_check_requirements {
 
-my $req = shift || undef;
-my $opt = shift || undef;
-my $appname = shift || undef;
+my $req = shift;
+my $opt = shift;
+my $appname = shift;
 
 my ($req2,$opt2) = (undef,undef);
 $req2 = $req->{$appname} if (defined $req and defined $appname);
@@ -524,6 +541,7 @@ if (defined $opt2) {
 This function checks existence of a command and return its full pathname or undef if not found.
 The command name is passed as first parameter.
 The second parameter should be 0 if the command is mandatory, 1 if optional.
+It returns the full path name of the command if found, undef otherwise and dies if that was a mandatory command
 
 =cut
 
